|##########|
|#MAGIC   #|BNDMBPCA
|#PROJECT #|"OGfxDemo"
|#PATHS   #|"StdProject"
|#FLAGS   #|xx-x-x--x---x-x-----------------
|#USERSW  #|--------------------------------
|#USERMASK#|--------------------------------
|#SWITCHES#|x----xxxxx-xx---
|##########|
MODULE OGfxDemo;
| Dieses Demo zeigt objektorientierte Elemente des neuen Clusters (V1.4)
| anhand graphischer Elemente.

FROM GfxScreen IMPORT ScreenGrp,ColorGrp;
FROM GfxDraw   IMPORT DrawCircle,DrawRectangle,AreaCircle,AreaRectangle;
FROM GfxInput  IMPORT UserActions,WaitUser;
               IMPORT OTrees;

VAR scr : Screen;   | Zugriff auf Ausgabeschirm

TYPE
  GObject     = POINTER TO GObjectObj;   | Basistyp fr alle graphischen Objekte

DEFINITION MODULE GObjTree = OTrees.StdTrees(GObject); | Ausprgung des Baum-
                                                       | moduls fr unseren
                                                       | speziellen Typ

TYPE
  GObjectObj  = OBJECT OF GObjTree.Tree;
                  dispX,dispY : INTEGER;
                  CONSTRUCTOR Create(dispX,
                                     dispY : INTEGER;
                                     sons  : LIST OF GObject):GObject;

                  METHOD DrawYourself(atX,atY : INTEGER);
                  METHOD AreaDrawYourself(atX,atY : INTEGER);

                  METHOD Half;
                  METHOD Double;
                END;


| Create erzeugt ein GObjekt an der Position (dispX/dispY), mit einer Liste
| abhngiger Subobjekte.

METHOD GObject.Create(dispX,
                      dispY : INTEGER;
                      sons  : LIST OF GObject):GObject;
VAR i : INTEGER;
BEGIN
  SUPER.Create;               | Fhre Create-Constructor des Vorfahren aus
  SELF.dispX:=dispX;          | Trage Position ein
  SELF.dispY:=dispY;          |
  FOR i:=0 TO sons'MAX DO     | Trage Shne in die eigene Liste ein.
    SELF.InsertLast(sons[i]); |
  END;
  RETURN SELF;                | Gebe Verweis auf sich selbst zurck, erleichtert
END Create;                   | die Konstruktion von neuen Objekten s.u.



| DrawYourself
| Lassen Sie sich nicht verwirren, da keine eigentlichen Zeichenoperationen
| ausgefhrt werden; diese Methoden werden beim Erben noch berdefiniert.

METHOD GObjectObj.DrawYourself(atX,atY : INTEGER);

  PROCEDURE DrawSon(t : GObject);
  BEGIN
    t.DrawYourself(atX+dispX,atY+dispY);
  END DrawSon;

BEGIN
  SELF.Apply(DrawSon); | Rufe fr jedes Sohnobjekt DrawYourself auf,
                       | ist bei einem geerbten Objekt diese Funktion
                       | berdefiniert, wird die neue Methode an deren
                       | Stelle aufgerufen.
END DrawYourself;


| AreaDrawYourself wie DrawYourself, jedoch mit ausgefllten Flchen.
METHOD GObjectObj.AreaDrawYourself(atX,atY : INTEGER);

  PROCEDURE DrawSon(t : GObject);
  BEGIN
    t.AreaDrawYourself(atX+dispX,atY+dispY);
  END DrawSon;

BEGIN
  SELF.Apply(DrawSon);
END AreaDrawYourself;

METHOD GObject.Half;  | Halbiert die Position eines GObjekt und seiner Shne

  PROCEDURE HalfSon(t : GObject);
  BEGIN
    t.Half;
  END HalfSon;

BEGIN
  dispX:=dispX DIV 2;
  dispY:=dispY DIV 2;
  SELF.Apply(HalfSon);
END Half;

METHOD GObject.Double; | Verdoppelt die Position eines GObjects und seiner Shne

  PROCEDURE DoubleSon(t : GObject);
  BEGIN
    t.Double;
  END DoubleSon;

BEGIN
  dispX:=dispX * 2;
  dispY:=dispY * 2;
  SELF.Apply(DoubleSon);
END Double;


TYPE
| Definition eines speziellen GObjects durch Erben

  OCircle     = POINTER TO OBJECT OF GObject;
                  radius : INTEGER;
                  CONSTRUCTOR Create(dispX,dispY,radius : INTEGER):GObject;

                  METHOD DrawYourself(atX,atY : INTEGER);
                  METHOD AreaDrawYourself(atX,atY : INTEGER);

                  METHOD Half;
                  METHOD Double;
                END;

METHOD OCircle.Create(dispX,dispY,radius : INTEGER):GObject;
BEGIN
  FORGET SUPER.Create(dispX,dispY); | Aufruf der Erzeugungsprozedur des
                                    | Vorfahren
  SELF.radius:=radius;              | Eintragung der spezifischen Werte
  RETURN SELF;
END Create;


|berdefinition von GObject.DrawYourself
METHOD OCircle.DrawYourself(atX,atY : INTEGER);
BEGIN
  DrawCircle(scr,dispX+atX,dispY+atY,radius); | Eigentliche Zeichenroutine
END DrawYourself;

|Das Gleiche nochmal fr ausgefllte Kreise
METHOD OCircle.AreaDrawYourself(atX,atY : INTEGER);
BEGIN
  AreaCircle(scr,dispX+atX,dispY+atY,radius);
END AreaDrawYourself;

METHOD OCircle.Half;
BEGIN
  SUPER.Half;          | Halbiere die Position, durch Aufruf der Vorfahren-
                       | methode
  radius:=radius DIV 2;| Halbiere Radius
END Half;


METHOD OCircle.Double;
BEGIN
  SUPER.Double;
  radius:=radius * 2;
END Double;

TYPE
| Noch ein weiteres spezielles Graphikobjekt, ein Rechteck,
| durch Erben aus einem GObject erzeugt

  ORectangle  = POINTER TO OBJECT OF GObject;
                  width,height : INTEGER;
                  CONSTRUCTOR Create(dispX,dispY,
                                     width,height : INTEGER):GObject;

                  METHOD DrawYourself(atX,atY : INTEGER);
                  METHOD AreaDrawYourself(atX,atY : INTEGER);

                  METHOD Half;
                  METHOD Double;
                END;

| Siehe OCircle.Create

METHOD ORectangle.Create(dispX,dispY,width,height : INTEGER):GObject;
BEGIN
  FORGET SUPER.Create(dispX,dispY);
  SELF.width:=width;
  SELF.height:=height;
  RETURN SELF;
END Create;

METHOD ORectangle.DrawYourself(atX,atY : INTEGER);
BEGIN
  DrawRectangle(scr,atX+dispX,atY+dispY,
                    atX+dispX+width,atY+dispY+height);
END DrawYourself;

METHOD ORectangle.AreaDrawYourself(atX,atY : INTEGER);
BEGIN
  AreaRectangle(scr,atX+dispX,atY+dispY,
                    atX+dispX+width,atY+dispY+height);
END AreaDrawYourself;

METHOD ORectangle.Half;
BEGIN
  SUPER.Half;
  width:=width DIV 2;
  height:=height DIV 2;
END Half;

METHOD ORectangle.Double;
BEGIN
  SUPER.Double;
  width:=width * 2;
  height:=height * 2;
END Double;

VAR man : GObject; | Variable vom Typ GObjekt

BEGIN
  OpenScreen(scr,1,TRUE,TRUE);  | ffne Ausgabeschirm
  IF scr#NIL THEN               | Erzeuge "man" mit Sohnobjekten
    man:=GObject.Create(0,0,
           OCircle.Create(80,20,20),
           ORectangle.Create(60,40,40,100),
           OCircle.Create(60,50,10),
           ORectangle.Create(20,40,40,20),
           OCircle.Create(20,50,10),
           ORectangle.Create(10,50,20,40),
           OCircle.Create(100,50,10),
           ORectangle.Create(100,40,40,20),
           OCircle.Create(140,50,10),
           ORectangle.Create(130,50,20,40)
           );
    man.DrawYourself(0,0);
    man.AreaDrawYourself(300,20);
    man.Half;
    man.DrawYourself(100,100);
    man.Double;
    man.DrawYourself(200,200);
    WaitUser(scr,{userMouse});
    CloseScreen(scr);
  END;
END OGfxDemo.

