|##########|
|#MAGIC   #|BNDMBDJK
|#PROJECT #|"Puzzle"
|#PATHS   #|"StdProject"
|#FLAGS   #|xx-x-x--x---x-x-----------------
|#USERSW  #|--------------------------------
|#USERMASK#|--------------------------------
|#SWITCHES#|xx---xxxxx------
|##########|
MODULE Puzzle;

(* $V- $R- $S- $N- *) | Compilerswitches

FROM GfxScreen   IMPORT Screen,OpenScreen,Palette,SetPalette,
                        FadeIn,FadeOut;
FROM GfxDraw     IMPORT ClearScreen,SetDrawMode,DrawModes,
                        AreaRectangle,SetAPen;
FROM GfxShape    IMPORT Shape,GetShape,PutShape,FreeShape,Sync;
FROM GfxText     IMPORT Font,OpenFont,SetFont,CharWidth;
FROM GfxPseudo3D IMPORT Write3D,Box3D,Color3D,Circle3D;
FROM GfxInput    IMPORT UserActions,UserAct,WaitUser,MouseClick,
                        CheckUser;
FROM Random      IMPORT RND;

CONST
  PuzzleTop   = 40; | Abstand vom oberen Bildschirmrand zum Puzzle

VAR
  GameScreen,
  BackScreen  : Screen; | Zugriffe auf die beiden Screens
  font        : Font;   | Zugriff auf den Zeichensatz
  chips       : ARRAY [0..15] OF Shape; | Feld mit Zugriffen auf
                                        | einen Shape.
  backfields  : ARRAY [0..3],[0..3] OF Shape; | Hier werden die einzelnen
                                              | Teile des Hintergrundes
                                              | abgelegt.
  Field       : ARRAY [0..3],
                      [0..3] OF SHORTINT; | In dieser Tabelle wird
                                          | die aktuelle Position der
                                          | einzelnen Puzzelteile eingetragen
  FreeX,FreeY : INTEGER;| Position des freien Feldes

PROCEDURE CreateChips;  | Erzeugt die einzelnen Puzzleteile
CONST ChipChars = ARRAY OF CHAR:("0","1","2","3", | Beschriftung der
                                 "4","5","6","7", | einzelnen Zeichen.
                                 "8","9","A","B", | Die Zeichen werden
                                 "C","D","E","F");| in einem Array abgelegt,
                                                  | damit man das Erzeugen
                                                  | der Teile in einer
                                                  | Schleife abhandeln kann.
      ChipColor = Color3D:(topLeft=1,     | Farben Puzzelteils,
                           bottomRight=3, | sie sind so gewhlt,
                           normal=2);     | da die Teile erhaben wirken.

      CharColor = Color3D:(topLeft=3,     | Die Farbe der Schrift
                           bottomRight=1, | dagegen bewirkt, da
                           normal=0);     | die Schrift wie ausgestanzt
                                          | wirkt.
VAR
  c  : CHAR;
  i  : INTEGER; | Schleifenzhler

BEGIN
  FOR i:=0 TO 15 DO | 16 Teile sollen gesetzt werden.
    Box3D(BackScreen,ChipColor,0,0,31,31); | Hier wird erstmal ein
                                           | Rechteck auf die Hilfs
                                           | screen gemalt.
    c:=ChipChars[i];
    Write3D(BackScreen,CharColor,          | Schreibt ein Zeichen auf das
                                           | Rechteck
            16-CharWidth(font,c) DIV 2,24, | Position, an die geschrieben
                                           | wird. Dabei wird mittels
                                           | "Charwidth" dafr gesorgt,
                                           | da die Zeichen zentriert
                                           | auf den Puzzelteilen erscheinen,
                                           | auch wenn ein Proportionaler
                                           | Zeichensatz verwendet wird.
                       c); | Zeichen, das geschrieben werden soll

    GetShape(BackScreen,chips[i],0,0,31,31); | Kopiert den Bereich, in
                                             | den gezeichnet worden ist,
                                             | und merkt sich einen
                                             | Verweis darauf in dem
  END;                                       | Array "chips"
END CreateChips;

PROCEDURE CreateBackground;
CONST
  BackColor = Color3D:(topLeft=4,bottomRight=6,normal=5);
VAR
  x,y : INTEGER;
BEGIN
  SetAPen(BackScreen,5); | Setze Hintergrundfarbe
  AreaRectangle(BackScreen,0,0,127,127); | Zeichne ein ausgeflltes
                                         | Rechteck in dieser Farbe
                                         | als Hintergrund
  | Die folgenden Schleifen malen nun aud den Hintergrund
  | 4 Kreise die vertieft liegend wirken
  FOR x:=14 TO 123 BY 20 DO
    FOR y:=14 TO 123 BY 20 DO
      Circle3D(BackScreen,BackColor,x,y,7);
    END;
  END;
  | Die Folgenden  Schleifen sichern jetzt jeweils ein Viereck
  | von der Gre eines Puzzleteils, diese werden gebraucht,
  | um spter beim Verschieben der Puzzleteile den Hintergrund
  | wieder herstellen zu knnen.
  FOR x:=0 TO 3 DO
    FOR y:=0 TO 3 DO
      GetShape(BackScreen,backfields[x,y],
               x*32,y*32,x*32+31,y*32+31);
    END;
  END;
END CreateBackground;

PROCEDURE DrawPicture; | Malt den Rahmen des Spielfeldes,
                       | indem ein vertieft wirkendes Rechteck
                       | in ein erhabenes gezeichnet wird.
CONST Frame1 = Color3D:(topLeft=7,bottomRight=9,normal=8);
      Frame2 = Color3D:(topLeft=9,bottomRight=7,normal=8);
VAR x,y : INTEGER;
BEGIN
  Box3D(GameScreen,Frame1,0,PuzzleTop,159,PuzzleTop+159);
  Box3D(GameScreen,Frame2,14,PuzzleTop+14,145,PuzzleTop+145);
END DrawPicture;

PROCEDURE InitField;| Gibt jedem Feld eine Steinnummer
VAR i : INTEGER;
BEGIN
  FOR i:=0 TO 14 DO
    Field[i MOD 4,i DIV 4]:=i;
  END;
  FreeX:=3;FreeY:=3;| Setzt das anfangs leere Feld.
END InitField;


PROCEDURE DrawField; | Zeichnet die Puzzleteile auf den Spielscreen
VAR x,y : INTEGER;
BEGIN
  SetDrawMode(GameScreen,drawAPen); | Setzt normalen Zeichenmodus
  FOR x:=0 TO 3 DO | Jede Spalte
    FOR y:=0 TO 3 DO | Jede Zeile
      PutShape(GameScreen,backfields[x,y],  | Setzt die Hintergrund
               x*32+16,y*32+(PuzzleTop+16));| Kacheln
      IF (x#FreeX) OR (y#FreeY) THEN | Wenn die Position ungleich
                                     | der Leeren ist,
        PutShape(GameScreen,chips[Field[x,y]],| Zeichne das
                 x*32+16,y*32+(PuzzleTop+16));| Puzzleteil, das
                                              | sich gerade an dieser
                                              | Position befindet
      END;
    END;
  END;
END DrawField;

CONST
  | Dieses Array wird dazu bentigt, um eine Realistischere
  | Bewegung zu erzielen.
  Para    = ARRAY OF SHORTINT:(1,3,6,10,14,19,24,29,32,
                               29,27,28,30,32,31,32);

| Bewegt den x. Stein in der Zeile, in der sich der freie
| Platz befindet nach rechts
PROCEDURE MoveRight(x : INTEGER);
VAR i,j : INTEGER;
    s   : Shape;
BEGIN
  FOR i:=0 TO Para'MAX DO | Innerhalb dieser Schleife
                          | werden die Puzzleteile stckchenweise
                          | verschoben.

    FOR j:=x TO FreeX DO | Baut auf der Hilfsscreen den Hintergrund
                         | des Bereiches auf, der vom Verschieben
                         | betroffen ist.
      PutShape(BackScreen,backfields[j,FreeY],j*32,0);
    END;
    FOR j:=x TO FreeX-1 DO | Setzt die Chips, die verschoben werden
                           | umd den Betrag "Para[i]" verschoben
                           | auf das Hintergrundmuster auf dem
                           | Hilfsscreen
      PutShape(BackScreen,chips[Field[j,FreeY]],j*32+Para[i],0);
    END;
    GetShape(BackScreen,s,
             x*32,0,FreeX*32+31,31);| Schneidet den neu gezeichneten
                                    | Bereich aus der Hilfsscreen aus
    Sync(GameScreen); | Sorgt dafr, da der Bildschirm dabei nicht
                      | flackert.
    PutShape(GameScreen,s,
             x*32+16,FreeY*32+(PuzzleTop+16)); | Und Kopiert ihn
                                               | an die richtige
                                               | Stelle auf den Spielscreen
    FreeShape(s); | Gibt den Shape wieder frei.
  END;
  FOR j:=FreeX TO x+1 BY -1 DO  | Verschiebt die Puzzleteile
                                | auch innerhalb des Arrays, in dem
                                | deren Position abgelegt ist.
    Field[j,FreeY]:=Field[j-1,FreeY]
  END;
  FreeX:=x; | Setze neue freie x-Position
END MoveRight;

| Bewegt den x. Stein in der Zeile, in der sich der freie
| Platz befindet nach links.
| Beschreibung, siehe MoveRight
PROCEDURE MoveLeft(x : INTEGER);
VAR i,j : INTEGER;
    s   : Shape;
BEGIN
  FOR i:=0 TO Para'MAX DO
    FOR j:=FreeX TO x DO
      PutShape(BackScreen,backfields[j,FreeY],j*32,0);
    END;
    FOR j:=FreeX+1 TO x DO
      PutShape(BackScreen,chips[Field[j,FreeY]],j*32-Para[i],0);
    END;
    GetShape(BackScreen,s,FreeX*32,0,x*32+31,31);
    Sync(GameScreen);
    PutShape(GameScreen,s,FreeX*32+16,FreeY*32+(PuzzleTop+16));
    FreeShape(s);
  END;
  FOR j:=FreeX TO x-1 DO
    Field[j,FreeY]:=Field[j+1,FreeY]
  END;
  FreeX:=x;
END MoveLeft;

| Bewegt den y. Stein in der Spalte, in der sich der freie
| Platz befindet nach unten
| Beschreibung, siehe MoveRight
PROCEDURE MoveDown(y : INTEGER);
VAR i,j : INTEGER;
    s   : Shape;
BEGIN
  FOR i:=0 TO Para'MAX DO
    FOR j:=y TO FreeY DO
      PutShape(BackScreen,backfields[FreeX,j],0,j*32);
    END;
    FOR j:=y TO FreeY-1 DO
      PutShape(BackScreen,chips[Field[FreeX,j]],0,j*32+Para[i]);
    END;
    GetShape(BackScreen,s,0,y*32,31,FreeY*32+31);
    Sync(GameScreen);
    PutShape(GameScreen,s,FreeX*32+16,y*32+(PuzzleTop+16));
    FreeShape(s);
  END;
  FOR j:=FreeY TO y+1 BY -1 DO
    Field[FreeX,j]:=Field[FreeX,j-1]
  END;
  FreeY:=y;
END MoveDown;

| Bewegt den y. Stein in der Spalte, in der sich der freie
| Platz befindet nach oben
| Beschreibung, siehe MoveRight
PROCEDURE MoveUp(y : INTEGER);
VAR i,j : INTEGER;
    s   : Shape;
BEGIN
  FOR i:=0 TO Para'MAX DO
    FOR j:=FreeY TO y DO
      PutShape(BackScreen,backfields[FreeX,j],0,j*32);
    END;
    FOR j:=FreeY+1 TO y DO
      PutShape(BackScreen,chips[Field[FreeX,j]],0,j*32-Para[i]);
    END;
    GetShape(BackScreen,s,0,FreeY*32,31,y*32+31);
    Sync(GameScreen);
    PutShape(GameScreen,s,FreeX*32+16,FreeY*32+(PuzzleTop+16));
    FreeShape(s);
  END;
  FOR j:=FreeY TO y-1 DO
    Field[FreeX,j]:=Field[FreeX,j+1]
  END;
  FreeY:=y;
END MoveUp;

PROCEDURE MixUpField | Mischt das Spiel zufllig
VAR i : INTEGER;
BEGIN
  FOR i:=1 TO 64 DO | es werden 65 Mischversuche unternommen
    IF KEY RND(4)  | Ziehe eine Zufallszahl zwischen 0 und 3
      OF 0 AND_IF FreeX>0 THEN | Wenn die freie Stelle nicht am linken
                               | Rand liegt,
             MoveRight(RND(FreeX))| bewege einen Stein zwischen dem linken
                                  | Rand und der freien Stelle nach rechts.
                                  | Eventuell dazwischenliegende Steine
                                  | werden mitverschoben.
           END
      OF 1 AND_IF FreeX<3 THEN | Wenn die freie Stelle nicht am rechten
                               | Rand liegt,
             MoveLeft(3-RND(3-FreeX))| bewege einen Stein zwischen dem rechten
                                     | Rand und der freien Stelle nach links.
           END
      OF 2 AND_IF FreeY>0 THEN | Entsprechendes fr oben und unten.
             MoveDown(RND(FreeY))
           END
      OF 3 AND_IF FreeY<3 THEN
             MoveUp(3-RND(3-FreeY))
           END
    END;
  END;
END MixUpField;

VAR
  x,y : INTEGER; | Hierin werden spter die Koordinaten des letzten
                 | Mausklick abgelegt.

BEGIN
  OpenScreen(BackScreen,4,FALSE,FALSE); | Hier wird eine Screen mit
                                        | 16 Farben mit den Auflsungen:
                                        | 320x256 geffnet. Er wird noch fr das
                                        | Ablegen der Puzzleteile bentigt.
  SetDrawMode(BackScreen,drawAPen);     | Zeichenmodus auf normales Malen
                                        | einstellen.
  OpenScreen(GameScreen,4,FALSE,FALSE); | Hier wird der Spielscreen geffnet

  | Die nchste Anweisung setzt alle Farben des Screens auf Schwarz.
  SetPalette(GameScreen,Palette:((0,0,0),(0,0,0),(0,0,0),(0,0,0),
                                 (0,0,0),(0,0,0),(0,0,0),(0,0,0),
                                 (0,0,0),(0,0,0),(0,0,0),(0,0,0),
                                 (0,0,0),(0,0,0),(0,0,0),(0,0,0)));
  OpenFont(font,"diamond",20); | Hier ffnen wir einen eigenen Zeichensatz.
  SetFont(BackScreen,font);    | Setze den Font fr den Backscreen.
  CreateChips;                 | Erzeuge Puzzleteile.
  CreateBackground;            | Erzeuge Puzzlehintergrund.
  DrawPicture;                 | Zeichne Puzzlerahmen.
  InitField;DrawField;         | Initialisiere Puzzle und zeichne es.
  FadeIn(GameScreen,Palette:((0,0,0),
                             (0,12,15),(0,8,10),(0,4,5),
                             (0, 0,15),(0,0,10),(0,0,5),
                             (15,0,0),(10,0,0),(5,0,0)),16);
  MixUpField; | Mische Puzzle
  REPEAT | Diese Schleife wird solange ausgefhrt,
         | bis eine Taste gedrckt wird.
    WaitUser(GameScreen,UserAct:{userKey,userMouse});| Wartet auf
                                                     | eine Eingabe
    IF CheckUser(GameScreen,UserAct:{userMouse}) THEN
      | Falls die Eingabe ein Mausklick war,
      | wird die Position des Klicks abgefragt.
      MouseClick(GameScreen,x,y); | Dies geschieht hier.
      x:=(x-16) DIV 32;            | Welche Spalte im Puzzle ?
      y:=(y-(PuzzleTop+16)) DIV 32;| Welche Reiche ?
      IF (x OF 0..3) AND (y OF 0..3) THEN | Check, ob der Klick
                                          | innerhalb des Puzzles
                                          | stattfand.
        IF x=FreeX | Wenn der Stein in der angeklickten Spalte bewegt
                   | werden kann,
          AND_IF y>FreeY THEN MoveUp(y)| Und y grer als die nchste
                                       | freie Position ist, bewege
                                       | den Stein nach oben (bedenken
                                       | Sie dabei, da Bildschirmkoordinaten
                                       | nach oben hin abnehmen.
          OR_IF  y<FreeY THEN MoveDown(y) | Im anderen Fall wird nach
                                          | unten bewegt
          END
        OR_IF y=FreeY | Falls der Stein nur in der gewhlten Reihe
                      | bewegt werden kann,
          AND_IF x>FreeX THEN MoveLeft(x) | und sich der freie Platz
                                          | links von der Klickposition
                                          | befindet, bewege den Stein
                                          | nach links.
          OR_IF  x<FreeX THEN MoveRight(x)| Sonst nach rechts.
          END
        END;
      END;
    END;
  UNTIL CheckUser(GameScreen,UserAct:{userKey});
  FadeOut(GameScreen,16); | Ausblenden des Screens
CLOSE
  | Normalerweise sollte man hier den Screen und den Font wieder
  | Schlieen bzw. freigeben, da die Gfx-Module dies jedoch von selbst
  | am Programmende machen, knnen wir darauf verzichten
END Puzzle.

